var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const vue = require("@volar/vue-language-core");
const vueTs = require("@volar/vue-typescript");
const path = require("path");
const tsFaster = require("@volar/typescript-faster");
const init = (modules) => {
    const { typescript: ts } = modules;
    const vueFilesGetter = new WeakMap();
    const pluginModule = {
        create(info) {
            // fix: https://github.com/johnsoncodehk/volar/issues/1146
            if (info.project.projectKind === ts.server.ProjectKind.Inferred) {
                return info.languageService;
            }
            const proxyHost = createProxyHost(ts, info);
            if (proxyHost.getVueFiles().length === 0) {
                return info.languageService;
            }
            // fix: https://github.com/johnsoncodehk/volar/issues/205
            info.project.getScriptKind = fileName => {
                switch (path.extname(fileName)) {
                    case '.vue': return ts.ScriptKind.Deferred;
                    case '.js': return ts.ScriptKind.JS;
                    case '.jsx': return ts.ScriptKind.JSX;
                    case '.ts': return ts.ScriptKind.TS;
                    case '.tsx': return ts.ScriptKind.TSX;
                    case '.json': return ts.ScriptKind.JSON;
                    default: return ts.ScriptKind.Unknown;
                }
            };
            const ls = vueTs.createLanguageService(proxyHost.host);
            tsFaster.decorate(ts, proxyHost.host, ls);
            vueFilesGetter.set(info.project, proxyHost.getVueFiles);
            return new Proxy(info.languageService, {
                get: (target, property) => {
                    if (property === 'getSemanticDiagnostics'
                        || property === 'getEncodedSemanticClassifications'
                        || property === 'getCompletionsAtPosition'
                        || property === 'getCompletionEntryDetails'
                        || property === 'getCompletionEntrySymbol'
                        || property === 'getQuickInfoAtPosition'
                        || property === 'getSignatureHelpItems'
                        || property === 'getRenameInfo'
                        || property === 'findRenameLocations'
                        || property === 'getDefinitionAtPosition'
                        || property === 'getDefinitionAndBoundSpan'
                        || property === 'getTypeDefinitionAtPosition'
                        || property === 'getImplementationAtPosition'
                        || property === 'getReferencesAtPosition'
                        || property === 'findReferences') {
                        return ls[property];
                    }
                    return target[property];
                },
            });
        },
        getExternalFiles(project) {
            const getVueFiles = vueFilesGetter.get(project);
            if (!getVueFiles) {
                return [];
            }
            return getVueFiles().filter(fileName => project.fileExists(fileName));
        },
    };
    return pluginModule;
};
function createProxyHost(ts, info) {
    let projectVersion = 0;
    let reloadVueFilesSeq = 0;
    let sendDiagSeq = 0;
    let disposed = false;
    const extraFileExtensions = [{
            extension: 'vue',
            isMixedContent: true,
            scriptKind: ts.ScriptKind.Deferred,
        }];
    const vueFiles = new Map();
    const host = {
        getNewLine: () => info.project.getNewLine(),
        useCaseSensitiveFileNames: () => info.project.useCaseSensitiveFileNames(),
        readFile: path => info.project.readFile(path),
        writeFile: (path, content) => info.project.writeFile(path, content),
        fileExists: path => info.project.fileExists(path),
        directoryExists: path => info.project.directoryExists(path),
        getDirectories: path => info.project.getDirectories(path),
        readDirectory: (path, extensions, exclude, include, depth) => info.project.readDirectory(path, extensions, exclude, include, depth),
        realpath: info.project.realpath ? path => info.project.realpath(path) : undefined,
        getCompilationSettings: () => info.project.getCompilationSettings(),
        getVueCompilationSettings: () => { var _a; return (_a = parsedCommandLine === null || parsedCommandLine === void 0 ? void 0 : parsedCommandLine.vueOptions) !== null && _a !== void 0 ? _a : {}; },
        getCurrentDirectory: () => info.project.getCurrentDirectory(),
        getDefaultLibFileName: () => info.project.getDefaultLibFileName(),
        getProjectVersion: () => info.project.getProjectVersion() + '-' + projectVersion,
        getProjectReferences: () => info.project.getProjectReferences(),
        getScriptFileNames,
        getScriptVersion,
        getScriptSnapshot,
        getTypeScriptModule: () => ts,
        isTsPlugin: true,
    };
    update();
    const directoryWatcher = info.serverHost.watchDirectory(info.project.getCurrentDirectory(), onAnyDriveFileUpdated, true);
    const projectName = info.project.projectName;
    let tsconfigWatcher = info.project.fileExists(projectName)
        ? info.serverHost.watchFile(projectName, () => {
            onConfigUpdated();
            onProjectUpdated();
            parsedCommandLine = vue.createParsedCommandLine(ts, ts.sys, projectName, extraFileExtensions);
        })
        : undefined;
    let parsedCommandLine = tsconfigWatcher // reuse fileExists result
        ? vue.createParsedCommandLine(ts, ts.sys, projectName, extraFileExtensions)
        : undefined;
    return {
        host,
        getVueFiles: () => [...vueFiles.keys()],
        dispose,
    };
    function onAnyDriveFileUpdated(fileName) {
        return __awaiter(this, void 0, void 0, function* () {
            if (fileName.endsWith('.vue') && info.project.fileExists(fileName) && !vueFiles.has(fileName)) {
                onConfigUpdated();
            }
        });
    }
    function onConfigUpdated() {
        return __awaiter(this, void 0, void 0, function* () {
            const seq = ++reloadVueFilesSeq;
            yield sleep(100);
            if (seq === reloadVueFilesSeq && !disposed) {
                update();
            }
        });
    }
    function getScriptFileNames() {
        return info.project.getScriptFileNames().concat([...vueFiles.keys()]);
    }
    function getScriptVersion(fileName) {
        if (vueFiles.has(fileName)) {
            return vueFiles.get(fileName).version.toString();
        }
        return info.project.getScriptVersion(fileName);
    }
    function getScriptSnapshot(fileName) {
        if (vueFiles.has(fileName)) {
            const version = getScriptVersion(fileName);
            const file = vueFiles.get(fileName);
            if (file.snapshotsVersion !== version) {
                const text = getScriptText(fileName);
                if (text === undefined)
                    return;
                file.snapshots = ts.ScriptSnapshot.fromString(text);
                file.snapshotsVersion = version;
                return file.snapshots;
            }
            return file.snapshots;
        }
        return info.project.getScriptSnapshot(fileName);
    }
    function getScriptText(fileName) {
        if (info.project.fileExists(fileName)) {
            return info.project.readFile(fileName);
        }
    }
    function getVueFiles() {
        const parseConfigHost = {
            useCaseSensitiveFileNames: info.project.useCaseSensitiveFileNames(),
            readDirectory: (path, extensions, exclude, include, depth) => {
                return info.project.readDirectory(path, extensions.filter(ext => ext.endsWith('.vue')), exclude, include, depth);
            },
            fileExists: fileName => info.project.fileExists(fileName),
            readFile: fileName => info.project.readFile(fileName),
        };
        // fix https://github.com/johnsoncodehk/volar/issues/1276
        // Should use raw tsconfig json not rootDir but seems cannot get it from plugin info
        const includeRoot = path.resolve(info.project.getCurrentDirectory(), info.project.getCompilerOptions().rootDir || '.');
        const { fileNames } = ts.parseJsonConfigFileContent({}, parseConfigHost, includeRoot, info.project.getCompilerOptions(), undefined /* TODO: info.project.config.configFilePath? */, undefined, extraFileExtensions);
        return fileNames;
    }
    function update() {
        var _a;
        const newVueFiles = new Set(getVueFiles());
        let changed = false;
        for (const fileName of vueFiles.keys()) {
            if (!newVueFiles.has(fileName)) {
                (_a = vueFiles.get(fileName)) === null || _a === void 0 ? void 0 : _a.fileWatcher.close();
                vueFiles.delete(fileName);
                changed = true;
            }
        }
        for (const fileName of newVueFiles) {
            if (!vueFiles.has(fileName)) {
                const fileWatcher = info.serverHost.watchFile(fileName, (_, eventKind) => {
                    var _a;
                    if (eventKind === ts.FileWatcherEventKind.Changed) {
                        onFileChanged(fileName);
                    }
                    else if (eventKind === ts.FileWatcherEventKind.Deleted) {
                        (_a = vueFiles.get(fileName)) === null || _a === void 0 ? void 0 : _a.fileWatcher.close();
                        vueFiles.delete(fileName);
                        onProjectUpdated();
                    }
                });
                vueFiles.set(fileName, {
                    fileWatcher,
                    version: 0,
                    snapshots: undefined,
                    snapshotsVersion: undefined,
                });
                changed = true;
            }
        }
        if (changed) {
            onProjectUpdated();
        }
    }
    function onFileChanged(fileName) {
        fileName = path.resolve(fileName);
        const file = vueFiles.get(fileName);
        if (file) {
            file.version++;
        }
        onProjectUpdated();
    }
    function onProjectUpdated() {
        return __awaiter(this, void 0, void 0, function* () {
            projectVersion++;
            const seq = ++sendDiagSeq;
            yield sleep(100);
            if (seq === sendDiagSeq) {
                info.project.refreshDiagnostics();
            }
        });
    }
    function dispose() {
        directoryWatcher.close();
        if (tsconfigWatcher) {
            tsconfigWatcher.close();
        }
        for (const [_, file] of vueFiles) {
            file.fileWatcher.close();
        }
        disposed = true;
    }
}
function sleep(ms) {
    return new Promise(resolve => setTimeout(resolve, ms));
}
module.exports = init;
//# sourceMappingURL=index.js.map